clear all

%% 1. Load the sampling locations.
load('C:\MATLAB7\work\OUTPUT\all_locations.mat');
% all_locations = alt_loc_prev_segment;

%% 2. List the file in the Profile points and Joint Length folders
%%    joint length file contains coordinates of the joint and the cross
%%    sections location
profile_points_files_AF = dir(fullfile(matlabroot,'work\Profile Data\Profile points asian female\*.vda'))
joint_length_files_AF = dir(fullfile(matlabroot,'work\Profile Data\Joint length asian female\*.txt')) 

profile_points_files_AM = dir(fullfile(matlabroot,'work\Profile Data\Profile points asian male\*.vda'))
joint_length_files_AM = dir(fullfile(matlabroot,'work\Profile Data\Joint length asian male\*.txt')) 

profile_points_files_CF = dir(fullfile(matlabroot,'work\Profile Data\Profile points caucasian female\*.vda'))
joint_length_files_CF = dir(fullfile(matlabroot,'work\Profile Data\Joint length caucasian female\*.txt')) 

profile_points_files_CM = dir(fullfile(matlabroot,'work\Profile Data\Profile points caucasian male\*.vda'))
joint_length_files_CM = dir(fullfile(matlabroot,'work\Profile Data\Joint length caucasian male\*.txt')) 

%% 5. Reading file for all subject and find their distances.
%% 6. Substract the distances of the neutral profile with the template
%% 7. Substract the distance of the profiles for other key postures with
%%    corresponding neutral profile
%% 8. Delete the S-UAF and LAF-W for each key posture. Use this data to
%%    create PCA.

LAF_W = find(all_locations(2,:) == 6);
[unused col_LAF_W] = size(LAF_W);
S_UAF = find(all_locations(2,:) == 1);
[unused col_S_UAF] = size(S_UAF);

for i=1:4
    if i==1
        i
        profile_points_files = profile_points_files_AF;
        joint_length_files = joint_length_files_AF;
    elseif i == 2
        profile_points_files = profile_points_files_AM;
        joint_length_files = joint_length_files_AM;
    elseif i == 3
        profile_points_files = profile_points_files_CF;
        joint_length_files = joint_length_files_CF;
    else
        profile_points_files = profile_points_files_CM;
        joint_length_files = joint_length_files_CM;
    end
    
    
    [row_joint col] = size(joint_length_files);
    [row_profile col] = size(profile_points_files);
    for j = 1:row_joint %% to cover all subjects
        joint_length_per_subject = read_joint_length_files(joint_length_files(j).name,i);
    
        joint_length_per_subject(:,8) =  joint_length_per_subject(1,7)*2/3;
        joint_length_per_subject(:,9) =  joint_length_per_subject(1,7)*1/3;
        joint_length_per_subject(:,11) =  joint_length_per_subject(1,13)*1/3;
        joint_length_per_subject(:,12) =  joint_length_per_subject(1,13)*2/3;
    
        for k=1:(row_profile/row_joint)      
            joint_length_per_key_posture = joint_length_per_subject(k,7:13);
            arm_vector = joint_length_per_subject(k,1:6);
            [sampling_all]= vector_prev_segment(joint_length_per_key_posture,arm_vector,all_locations);
            if i == 1
                fid = fopen(char(fullfile(matlabroot,'work\Profile Data\Profile points asian female\',profile_points_files((j-1)*4 + k).name)));
            elseif i == 2
                fid = fopen(char(fullfile(matlabroot,'work\Profile Data\Profile points asian male\',profile_points_files((j-1)*4 + k).name)));
            elseif i == 3
                fid = fopen(char(fullfile(matlabroot,'work\Profile Data\Profile points caucasian female\',profile_points_files((j-1)*4 + k).name)));
            else
                fid = fopen(char(fullfile(matlabroot,'work\Profile Data\Profile points caucasian male\',profile_points_files((j-1)*4 + k).name)));
            end
            
            [coordinates] = read_txt_vda(fid);
            bezier = bspline_to_bezier(coordinates(:,2:3));
            sampled_points = multiple_sampling_new_version(bezier,sampling_all);   
            
            distance_vector = sampled_points-sampling_all(:,1:2);
            [row_sampled_points unused] = size(sampled_points);         

            
            for l=1:row_sampled_points
                profile_distance_per_subject(l,1) = (dot(distance_vector(l,:),distance_vector(l,:)))^0.5;% - template_distance(l,:);
            end
        
            [bezier_coordinates] = drawing_bezier(bezier);
            if k == 1
                profile_distance_all_per_subject = profile_distance_per_subject;
                profile_distance_FDA_all_per_subject = profile_distance_per_subject((col_S_UAF+2):(row_sampled_points-col_LAF_W-1),:);
            else
                profile_distance_current = profile_distance_per_subject;% - profile_distance_all_per_subject(1:row_sampled_points,1);
                profile_distance_FDA_current = profile_distance_current((col_S_UAF+2):(row_sampled_points-col_LAF_W-1),:);
                profile_distance_all_per_subject = [profile_distance_all_per_subject;profile_distance_current];
                profile_distance_FDA_all_per_subject = [profile_distance_FDA_all_per_subject;profile_distance_FDA_current];
            end
        end
    
        if j==1
            profile_distance_all =  profile_distance_all_per_subject;
            profile_distance_FDA_all = profile_distance_FDA_all_per_subject;
        else
            profile_distance_all = [profile_distance_all    profile_distance_all_per_subject]; 
            profile_distance_FDA_all = [profile_distance_FDA_all    profile_distance_FDA_all_per_subject];
        end
    end

    if i == 1
        save('C:\MATLAB7\work\OUTPUT\profile_distance_per_subject_original_AF.mat','profile_distance_all');
    elseif i == 2
        save('C:\MATLAB7\work\OUTPUT\profile_distance_per_subject_original_AM.mat','profile_distance_all');
    elseif i == 3
        save('C:\MATLAB7\work\OUTPUT\profile_distance_per_subject_original_CF.mat','profile_distance_all');
    else
        save('C:\MATLAB7\work\OUTPUT\profile_distance_per_subject_original_CM.mat','profile_distance_all');
    end
end
