clear all

%% ________________________________ WORLD CS ______________________________
%% 1. obtain vertices and coordinates from wrl file
%% 2. obtain joint from joint text file
%% 3. index y element of the coordinates
[coordinates,vertices] = extracting_wrl;
[input_skeleton] = extracting_joint;
[index_height] = indexing_height (coordinates);

%% 4. translation: E input = [0 0 0]
translation_vector = [1  0  0  0;0  1  0  0;0  0  1  0;-input_skeleton(2,1) -input_skeleton(2,2) -input_skeleton(2,3) 1];
[input_skeleton] = translation(translation_vector,input_skeleton);
[coordinates] = translation(translation_vector,coordinates);

%% 5. separate vertices for E (+1/4 and -1/4 from the E)
%% 6. separate upper and lower arm vertices
[start_partial,end_partial,vertices_partial] = partial_retrieve(vertices,coordinates);
[upper_vertices_partial,lower_vertices_partial] = upper_lower_retrieve(vertices,coordinates);

%% 7. obtain the index height that must be excluded for creating the front arm profile 
% [index_height_upper_exclusion] = profile_front_exclusion(upper_vertices_partial,index_height,coordinates);

%% ________________________________ SHOULDER ELBOW CS _____________________
load ('C:\MATLAB7\work\INITIAL INPUT\template_skeleton_NEWCS.mat');
load ('C:\MATLAB7\work\INITIAL INPUT\change_basis_toNEWCS.mat');
load ('C:\MATLAB7\work\INITIAL INPUT\change_basis_backtoCS.mat');

%% 8. transform the input skeleton into shoulder elbow CS
%% 9. transform the coordinates into shoulder elbow CS
input_skeleton=input_skeleton*change_basis_toNEWCS;
coordinates_NEWCS = coordinates*change_basis_toNEWCS;

%% 10.  create new CS for shoulder-elbow
cross_product = cross(input_skeleton(3,:),input_skeleton(1,:));
cross_product = cross_product/(dot(cross_product,cross_product)^0.5);
normal        = cross(input_skeleton(1,:),cross_product);
normal        = normal/(dot(normal,normal)^0.5);
SE            = input_skeleton(1,:)/(dot(input_skeleton(1,:),input_skeleton(1,:))^0.5);
CS_input_skeleton  = [normal;SE;cross_product];

%% 11.  calculate the rotation matrix for aligning shoulder-elbow with Y axis
%% 12.  transform the input skeleton and coordinates 
[Transform_toCS,Tranform_tooriginal] = CS_alignment(CS_input_skeleton);
input_skeleton = input_skeleton*Transform_toCS;
coordinates_NEWCS = coordinates_NEWCS*Transform_toCS;


%% 13.  calculate the normal from vertices adjacent to E
[normal] = calculate_normal(vertices_partial,coordinates_NEWCS);

%% 14.  calculate E CS and joints length
[length_UA_LA,E_unit] = calculate_UA_LA_E(input_skeleton);

%% 15.  create new basis using E CS
%% 16.  perform rotation of sampling rays in [1 0 0;0 1 0;0 0 1] and transform back to E CS
change_basis_backtoNEWCS = E_unit;
E_rays = zeros(32,3);
for i=1:32
    E_rays(i,:) = rotation_about_Y_axis([0 0 1],(i-1)*(2*pi/32))*change_basis_backtoNEWCS;
end

%% 17.  sampling E with the sampling rays
for i=1:32
    E_intersection_points(i,:) = sampling_WRL(E_rays(i,:),normal,vertices_partial,coordinates_NEWCS,[0 0 0]);
end



%% __________________________  ELBOW CS ___________________________________
load ('C:\MATLAB7\work\INITIAL INPUT\E_basis.mat');

%% 18. obtain rotation matrix by transforming E CS to template's E CS and match it with template's E CS
[Transform_toECS,Tranform_toNEWCS] = CS_alignment(E_unit*change_basis_toECS);

%% 19. transforming and rotating E sample points into E CS's template
E_intersection_points = E_intersection_points*change_basis_toECS*Transform_toECS;

E_intersection_points;

%% 20.  find the sample points where gradient is 0
[normal_E_flat_line] = spline_flat_line(E_intersection_points);

normal_E_flat_line;

%% 21.  calculate the angle between this point to [0 0 1]
cross_product = cross([0 0 1],normal_E_flat_line);

if cross_product (1,2)> 0
   dot_product = dot([0 0 1],normal_E_flat_line);
   dot_product = acos(dot_product)*180/pi;
else
   dot_product = dot([0 0 1],normal_E_flat_line); 
   dot_product = -acos(dot_product)*180/pi;
end


%% 22.  compare the angle with the template angle
if dot_product > 35.926
   dot_product-35.926;
   dot_product = (dot_product-35.926)*pi/180; 
   rotation_direction = -1;
elseif dot_product > 0 & dot_product < 35.926
   35.926 - dot_product;
   dot_product = (35.926 - dot_product)*pi/180; 
   rotation_direction = 1;
elseif dot_product < 0
   dot_product + 35.926;
   dot_product = (dot_product + 35.926)*pi/180; 
   rotation_direction = 1;
end

%% 23.  calculate the matrix for rotation
if rotation_direction > 0
    Y_rotation = [cos(dot_product)  0  -sin(dot_product);0      1    0;sin(dot_product)  0  cos(dot_product)];
else
    Y_rotation = [cos(-dot_product) 0  -sin(-dot_product);0      1    0;sin(-dot_product)  0  cos(-dot_product)];
end


%%% ________________________________ SHOULDER ELBOW CS _____________________
%% 24.  rotate the coordinates and input skeleton
coordinates_NEWCS = coordinates_NEWCS*Y_rotation;
input_skeleton = input_skeleton*Y_rotation;

% %% ________________________________ WORLD CS ______________________________
%% 24.  change the basis of the coordinates and input skeleton to the world CS
coordinates = coordinates_NEWCS*change_basis_backtoCS;
input_skeleton = input_skeleton*change_basis_backtoCS;

%% 25.  obtain the side profile
[profile_left,profile_right] = WRL_profile_side_version_2(coordinates,index_height);
%% 26.  modify and obtain the front profile
% [profile_in,profile_out,lower_point] = WRL_profile_front(coordinates,index_height,index_height_upper_exclusion);
[profile_in,profile_out,lower_point] = WRL_profile_front(coordinates,index_height);

%% 27.  prediction the flesh deformation area
[UAF_distance_neutral,LAF_distance_neutral,UAM_distance_neutral,LAM_distance_neutral,race] = flesh_deformation_area_version_2(profile_left, profile_right,input_skeleton,length_UA_LA);

%% 28.  determine local CS for upper/lower arm and elbow
[UAF_CS,E_CS,LAF_CS] = determine_local_CS (input_skeleton,length_UA_LA);

%% ___________________________ Cross section CS _________________________
%% 29.  sample the cross section and change them to 2D cross sections automatically
UAF_points = sampling_final(coordinates,upper_vertices_partial,UAF_distance_neutral*UAF_CS(2,:),UAF_CS)
UAM_points = sampling_final(coordinates,upper_vertices_partial,UAM_distance_neutral*UAF_CS(2,:),UAF_CS)
E_points = sampling_final(coordinates,vertices_partial,[1 1 1],E_CS)
LAM_points = sampling_final(coordinates,vertices,LAM_distance_neutral*LAF_CS(2,:),LAF_CS)
LAF_points = sampling_final(coordinates,lower_vertices_partial,LAF_distance_neutral*LAF_CS(2,:),LAF_CS)

%% 30.  completing missing points of UAF
[row col] = size(UAF_points);
list = find(abs(round(UAF_points(:,2))) > 0);
if isempty(list) == 0
   [UAF_points] = missing_UAM(UAF_points,list); 
end

%% 31.  completing missing points of UAM
[row col] = size(UAM_points);
list = find(abs(round(UAM_points(:,2))) > 0);
if isempty(list) == 0
   [UAM_points] = missing_UAM(UAM_points,list); 
end


save('C:\MATLAB7\work\OUTPUT\CS_points.mat','UAF_points','UAM_points','E_points','LAM_points','LAF_points');
save('C:\MATLAB7\work\OUTPUT\CS.mat','UAF_CS','E_CS','LAF_CS')
save('C:\MATLAB7\work\OUTPUT\profile.mat','profile_right','profile_left','profile_in','profile_out')
save('C:\MATLAB7\work\OUTPUT\skeleton_length.mat','length_UA_LA')
save('C:\MATLAB7\work\OUTPUT\race.mat','race')


